fct_beta_post = function(beta_prior,Q_prior,H,R,y,f_beta_nom,beta_nom) {
  #inputs and outputs: cf lecture slide 71
    iR = solve(R)
    beta_prior + solve(solve(Q_prior)+t(H)%*%iR%*%H)%*%t(H)%*%iR%*%
      (y - f_beta_nom - H%*%( beta_prior-beta_nom ) )
}

fct_Q_post = function(Q_prior,H,R) {
  #inputs and outputs: cf lecture slide 48
  iR = solve(R)
  solve(solve(Q_prior)+t(H)%*%iR%*%H)
}

fct_cov_gauss = function(x,y,sigma,l) {
  #x, y: nx-dim and ny-dim vectors if in dimension 1
  #      or nx*d and ny*d matrices if in dimensiond d>1
  #sigma: variance parameter
  #l: correlation length if d=1
  #   vector of correlation lengths if d>1
  #
  if (is.vector(x)) {  #Case d=1
    R = sigma^2 * exp( - outer(x/l,y/l,"-")^2 )
  }
  if (is.matrix(x)) {  #Case d>1
    d = dim(X)[2]
    n = dim(X)[1]
    R = matrix(nrow=n,ncol=n,data=sigma^2)
    for (i in 1:d) {
      R = R * exp( - outer(x[,i]/l[i],y[,i]/l[i],"-")^2 )
    }
  }
  R
}

fct_cov_matern_3_2 = function(x,y,sigma,l) {
  #x, y: nx-dim and ny-dim vectors if in dimension 1
  #      or nx*d and ny*d matrices if in dimensiond d>1
  #sigma: variance parameter
  #l: correlation length if d=1
  #   vector of correlation lengths if d>1
  #
  if (is.vector(x)) {  #Case d=1
    R = sigma^2 * (1+sqrt(6)*abs(outer(x/l,y/l,"-")))*
      exp( - sqrt(6)*abs(outer(x/l,y/l,"-")))
  }
  if (is.matrix(x)) {  #Case d>1
    d = dim(x)[2]
    nx = dim(x)[1]
    ny = dim(y)[1]
    R = matrix(nrow=nx,ncol=ny,data=sigma^2)
    for (i in 1:d) {
      R = R * (1+sqrt(6)*abs(outer(x[,i]/l[i],y[,i]/l[i],"-")))*
        exp( - sqrt(6)*abs(outer(x[,i]/l[i],y[,i]/l[i],"-")))
    }
  }
  R
}

fct_hat_phi = function(f_nom_x_0,h_0,beta_post,beta_nom,r_0,R,y,f_nom,H) {
  #inputs and outputs: cf lecture slide 73
  iR = solve(R)
  f_nom_x_0 + t(h_0)%*%(beta_post-beta_nom)+t(r_0)%*%iR%*%
    (y - f_nom - H%*%(beta_post-beta_nom))
}

fct_hat_sigma_2 = function(C_0,r_0,R,h_0,H,Q_prior) {
  #inputs and outputs: cf lecture slide 73
  iR = solve(R)
  C_0 - t(r_0)%*%iR%*%r_0 + t(h_0 - t(H)%*%iR%*%r_0)%*%
    solve(solve(Q_prior)+t(H)%*%iR%*%H)%*%(h_0 - t(H)%*%iR%*%r_0)
}
